/*
** log.c - part of GrabURL 2.0
** Copyright (C) 1996-97 Serge Emond
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "log.h"
#include "libgurl.h"
#include "macros.h"
#include "config.h"

char log_type[LOG_COUNT] = { '?', '*', '+', '>', 'X' };

struct Log *logs;

void putlog(const char *mesg, int type)
{
  char *p, *q, s_date[10], s_time[10];
  int i, std;
  time_t tt;
  struct tm *t;
  FILE *fh;
  struct Log *l;
  
  p = (char *)mesg;
  while (1) {
    q = strchr(p, '\n');
    if (q == 0) q = strchr(p, '\0');
    if ((q == 0) || (q == p)) break;
    
    tt = time(0);
    t = localtime(&tt);
    
    i = (int)q - (int)p;
    
    sprintf(s_date, "%02d/%02d/%02d ", t->tm_mday, t->tm_mon, t->tm_year);
    sprintf(s_time, "%02d:%02d:%02d ", t->tm_hour, t->tm_min, t->tm_sec);
    
    for (l = logs; l; l = l->next) {

      if (!str_ieq(l->name, "stdout")) {
	std=1;
	fh = stdout;
      }
      else if (!str_ieq(l->name, "stderr")) {
	std=1;
	fh = stderr;
      }
      else {
	std=0; fh=0;
      }

      if (std || ((fh = fopen(l->name, "a")) != 0)) {
	if (l->flags & LOGF_TYPE)
	  fprintf(fh, "%c ", (p==mesg ? log_type[type] : ' '));
	if (l->flags & LOGF_DATE)
	  fprintf(fh, (p==mesg ? s_date : "         "));
	if (l->flags & LOGF_TIME)
	  fprintf(fh, (p==mesg ? s_time : "         "));
	fprintf(fh, "%.*s\n", i, p);
	if (!std) fclose(fh);
      }
    }
    
    if (q[0]==0) break;
    p = q+1;
  }
}

void log_cfg_init()
{
  logs = 0;
}

/* Log line: "AddLog <filename> [<flags>]' */

char *log_cfg_addlog(const char *line)
{
  struct Log *l;
  char *arg;
  int i;
  
  arg = ExtractArgument(line, 1);
  if (!arg) return "Missing argument to AddLog";
  
  l = (struct Log *)gmalloc(sizeof(struct Log));
  
  l->next = logs;
  l->name = arg;
  l->flags = 0;
  
  for (i=2; (arg = ExtractArgument(line, i)) != 0; i++) {
    if (!str_ieq(arg, "date")) l->flags |= LOGF_DATE;
    else if (!str_ieq(arg, "time")) l->flags |= LOGF_TIME;
    else if (!str_ieq(arg, "type")) l->flags |= LOGF_TYPE;
    else if (arg[0] == '#') { free(arg); break; }
    else fprintf(stderr, "Warning: Argument \"%s\" to AddLog is invalid",
		 arg);
    free(arg);
  }
  
  logs = l;
  return 0;
}
